# -*- coding: utf-8 -*-
from __future__ import unicode_literals


"""
zh custom locale file.
"""

translations = {
    # Relative time
    "after": "{time}后",
    "before": "{time}前",
    # Date formats
    "date_formats": {
        "LTS": "Ah点m分s秒",
        "LT": "Ah点mm分",
        "LLLL": "YYYY年MMMD日ddddAh点mm分",
        "LLL": "YYYY年MMMD日Ah点mm分",
        "LL": "YYYY年MMMD日",
        "L": "YYYY-MM-DD",
    },
}
