# -*- coding: utf-8 -*-
from __future__ import unicode_literals


"""
fo custom locale file.
"""

translations = {
    # Relative time
    "after": "{0} aftaná",
    "before": "{0} áðrenn",
    # Ordinals
    "ordinal": {"other": "."},
    # Date formats
    "date_formats": {
        "LTS": "HH:mm:ss",
        "LT": "HH:mm",
        "LLLL": "dddd D. MMMM, YYYY HH:mm",
        "LLL": "D MMMM YYYY HH:mm",
        "LL": "D MMMM YYYY",
        "L": "DD/MM/YYYY",
    },
}
